% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{tw_empty_wikipedia_category_members}
\alias{tw_empty_wikipedia_category_members}
\title{A zero-rows tibble used internally when \code{tw_empty_wikipedia_category_members()} would not return any value.}
\format{
A data frame with 0 rows and 3 columns
}
\usage{
tw_empty_wikipedia_category_members
}
\description{
A zero-rows tibble used internally when \code{tw_empty_wikipedia_category_members()} would not return any value.
}
\keyword{datasets}
